/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.util.Enumeration;
import java.util.Set;
import java.util.HashSet;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import org.xmloperator.java.syntax.IllegalSyntaxException;
import org.xmloperator.java.syntax.JavaTokenizer;
import org.xmloperator.java.syntax.JavaTokenizerImpl;
import org.xmloperator.java.syntax.IdentifierListMap;
import org.xmloperator.java.syntax.CompilationUnitParser;
import org.xmloperator.java.skeleton.model.Skeleton;
import org.xmloperator.java.skeleton.model.SkeletonFactory;
import org.xmloperator.java.skeleton.impl.SkeletonImpl;
import org.xmloperator.java.skeleton.util.SkeletonBuilderFromSAX;
import org.xmloperator.java.skeleton.util.SkeletonBuilderFromSource;
import org.xmloperator.java.skeleton.util.SkeletonWriter;
import org.xmloperator.java.skeleton.util.SkeletonPackageMap;

public final class Compiler {
  private static final boolean TIME = false;
  private static final String[] DEFAULT_PRIMARY_PACKAGES =
      {"java", "javax", "org", "com", "net"};
  private static final String[] USAGE_DESCRIPTION = {
      "",
      "Arguments of org.xmloperator.java.skeleton.Compiler.main :",
      "    ( -v | -nw | -fi | -pu | -si )*",
      "    ( -ppi <primaryPackageIdentifier> )*",
      "    ( -import <import_skeleton.xml> )*",
      "    ( -output <output_skeleton.xml> )?",
      "    ( <source.java> | <sources.zip> | <directory> | <skeleton.xml> )*",
      "",
      "-General arguments :",
      "  -v  : verbose",
      "  -nw : no warning",
      "",
      "-Compilation arguments :",
      "  -ppi    : additional primary package identifier",
      "  -import : import skeleton pathname",
      "  -si     : add a first compilation pass for self import",
      "  -fi     : will produce a pure import skeleton",
      "",
      "-Output arguments :",
      "  -pu      : include package use",
      "  -output  : output skeleton pathname",
      ""};
  private Skeleton outputSkeleton;
  private boolean isWarningPrinted;
  private boolean forImport;
  private Set primaryPackageIdentifierSet;
  private IdentifierListMap
      fromPackageQualifiedIdentifierToClassAndInterfaceIdentifierList;

  private static final class BadArgumentException extends Exception {
    public BadArgumentException() {}
  }

  public static void main(String[] args) {
    try {
      if (args.length == 0)
        throw new BadArgumentException();
      boolean isVerbose = false;
      boolean isWarningPrinted = true;
      boolean isSelfImported = false;
      boolean forImport = false;
      boolean withPackageUse = false;
      String outputSkeletonPathName = null;
      Set primaryPackageIdentifierSet = new HashSet();
      Skeleton importSkeleton = new SkeletonImpl();
      for (int k = 0; k < DEFAULT_PRIMARY_PACKAGES.length; k++)
        primaryPackageIdentifierSet.add(DEFAULT_PRIMARY_PACKAGES[k]);
      int argIndex = 0;
      // "-xxx" options
      while (argIndex < args.length) {
        String argument = args[argIndex];
        if (!argument.startsWith("-"))
          break;
        argIndex++;
        argument = argument.substring(1);
        if (argument.equals("v"))
          isVerbose = true;
        else if (argument.equals("nw"))
          isWarningPrinted = false;
        else if (argument.equals("si"))
          isSelfImported = true;
        else if (argument.equals("fi"))
          forImport = true;
        else if (argument.equals("pu"))
          withPackageUse = true;
        else if (argument.equals("ppi")) {
          if (args.length <= argIndex)
            throw new BadArgumentException();
          String primaryPackageIdentifier = args[argIndex++];
          primaryPackageIdentifierSet.add(primaryPackageIdentifier);
        }
        else if (argument.equals("import")) {
          if (args.length <= argIndex)
            throw new BadArgumentException();
          String importSkeletonPathName = args[argIndex++];
          if (isVerbose)
            System.out.println("import " + importSkeletonPathName);
          readSkeleton(importSkeleton, importSkeletonPathName, isWarningPrinted);
        }
        else if (argument.equals("output")) {
          if (args.length <= argIndex || outputSkeletonPathName != null)
            throw new BadArgumentException();
          outputSkeletonPathName = args[argIndex++];
        }
        else
          throw new BadArgumentException();
      }
      // self import
      if (isSelfImported) {
        Compiler compiler = new Compiler(importSkeleton);
        for (int k = argIndex; k < args.length; k++) {
          String pathName = args[k];
          if (pathName.endsWith(".xml"))
            continue;
          if (isVerbose)
            System.out.println("import " + pathName);
          compiler.compile(new File(pathName));
        }
      }
      // compilation
      Skeleton outputSkeleton = new SkeletonImpl();
      {
        Compiler compiler = new Compiler(
            outputSkeleton, new SkeletonPackageMap(importSkeleton),
            primaryPackageIdentifierSet, isWarningPrinted, forImport);
        long time0_ms = System.currentTimeMillis();
        for (int k = argIndex; k < args.length; k++) {
          String pathName = args[k];
          if (isVerbose)
            System.out.println("compile " + pathName);
          if (pathName.endsWith(".xml"))
            readSkeleton(outputSkeleton, pathName, isWarningPrinted);
          else
            compiler.compile(new File(pathName));
        }
        if (TIME)
          System.out.println(
              "time = " + (System.currentTimeMillis() - time0_ms) + " ms");
      }
      // write output skeletons
      if (isVerbose)
        System.out.println("writing out");
      if (outputSkeletonPathName != null) {
        outputSkeleton.sort();
        boolean withReverseUse = !forImport;
        writeSkeleton(
            outputSkeleton, outputSkeletonPathName,
            withReverseUse, withPackageUse);
      }
      if (isVerbose)
        System.out.println("---end");
      System.exit(0);
    }
    catch (BadArgumentException exception) {
      for (int k = 0; k < USAGE_DESCRIPTION.length; k++)
        System.out.println(USAGE_DESCRIPTION[k]);
      System.exit(1);
    }
    catch (Throwable throwable) {
      throwable.printStackTrace(System.out);
      System.out.println(throwable);
      System.exit(1);
    }
  }

  private static final void readSkeleton(
      Skeleton skeleton, String skeletonPathname, boolean isWarningPrinted)
      throws IOException, SAXException, ParserConfigurationException {
    InputStream inputStream = new FileInputStream(skeletonPathname);
    try {
      SkeletonBuilderFromSAX.buildSkeleton(
          skeleton, skeletonPathname, isWarningPrinted, inputStream);
    }
    finally {
      inputStream.close();
    }
  }

  private static final void writeSkeleton(
      Skeleton skeleton, String skeletonPathname,
      boolean withReverseUse, boolean withPackageUse)
      throws FileNotFoundException, UnsupportedEncodingException  {
    OutputStream outputStream = new FileOutputStream(skeletonPathname);
    PrintWriter printWriter =
        new PrintWriter(new OutputStreamWriter(outputStream, "UTF8"));
    try {
      printWriter.println(
          "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>");
      SkeletonWriter.printSkeleton(
          printWriter, withReverseUse, withPackageUse, skeleton);
    }
    finally {
      printWriter.close();
    }
  }

  // Constructor for import only
  private Compiler(Skeleton importSkeleton) {
    this(importSkeleton, null, null, false, true);
  }

  // Constructor for compile
  private Compiler(
      Skeleton outputSkeleton,
      IdentifierListMap
      fromPackageQualifiedIdentifierToClassAndInterfaceIdentifierList,
      Set primaryPackageIdentifierSet,
      boolean isWarningPrinted, boolean forImport) {
    if (outputSkeleton == null)
      throw new IllegalArgumentException();
    this.outputSkeleton = outputSkeleton;
    this.fromPackageQualifiedIdentifierToClassAndInterfaceIdentifierList =
        fromPackageQualifiedIdentifierToClassAndInterfaceIdentifierList;
    this.primaryPackageIdentifierSet = primaryPackageIdentifierSet;
    this.isWarningPrinted = isWarningPrinted;
    this.forImport = forImport;
  }

  private void compile(File file) throws IOException, IllegalSyntaxException {
    if (file.isDirectory()) {
      File[] files = file.listFiles();
      for (int k = 0; k < files.length; k++) {
        file = files[k];
        String fileName = file.getName();
        if (file.isDirectory()
            || (file.isFile()
                && (fileName.endsWith(".java") || fileName.endsWith(".zip"))))
          this.compile(file);
      }
      return;
    }
    String pathName = file.getPath();
    if (pathName.endsWith(".zip")) {
      ZipFile zipFile = new ZipFile(file);
      try {
        Enumeration entries = zipFile.entries();
        while (entries.hasMoreElements()) {
          ZipEntry zipEntry = (ZipEntry) entries.nextElement();
          String entryName = zipEntry.getName();
          if (entryName.endsWith(".java"))
            this.compile(pathName + '/' + entryName,
                         zipFile.getInputStream(zipEntry));
        }
      }
      finally {
        zipFile.close();
      }
    }
    else
      this.compile(pathName, new FileInputStream(file));
  }

  private void compile(String pathName, InputStream inputStream)
      throws IOException, IllegalSyntaxException {
    try {
      JavaTokenizer javaTokenizer =
          new JavaTokenizerImpl(pathName, inputStream);
      CompilationUnitParser.parse(
          javaTokenizer,
          new SkeletonBuilderFromSource(this.outputSkeleton, this.forImport),
          this.fromPackageQualifiedIdentifierToClassAndInterfaceIdentifierList,
          this.primaryPackageIdentifierSet, this.isWarningPrinted);
    }
    finally {
      inputStream.close();
    }
  }
}
