/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.impl;

import org.xmloperator.java.skeleton.model.ClassOrInterface;
import org.xmloperator.java.skeleton.model.ClassAndInterfaceList;
import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.IdentifiedPackage;
import org.xmloperator.java.skeleton.impl.ClassAndInterfaceListImpl;

public final class ClassOrInterfaceImpl implements ClassOrInterface {
  private final Package parentPackage;
  private final String identifier;
  private String qualifiedIdentifier = null;
  private boolean isDefined = false;
  private final ClassAndInterfaceList usedTypes =
      new ClassAndInterfaceListImpl((ClassOrInterface)this);
  private final ClassAndInterfaceList userTypes =
      new ClassAndInterfaceListImpl((ClassOrInterface)this);

  public ClassOrInterfaceImpl(Package parentPackage, String identifier) {
    if (parentPackage == null
        || identifier.length() == 0 || identifier.indexOf('.') >= 0)
      throw new IllegalArgumentException();
    this.parentPackage = parentPackage;
    this.identifier = identifier;
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    return object instanceof ClassOrInterface
        && this.getQualifiedIdentifier().
        equals(((ClassOrInterface)object).getQualifiedIdentifier());
  }

  //////////////////////////////////
  // ClassOrInterface implementation
  //////////////////////////////////

  public Package getPackage() {
    return this.parentPackage;
  }

  public String getIdentifier() {
    return this.identifier;
  }

  public String getQualifiedIdentifier() {
    if (this.qualifiedIdentifier == null)
      this.qualifiedIdentifier =
          this.parentPackage.isDefault() ? this.identifier :
          ((IdentifiedPackage)this.parentPackage).getQualifiedIdentifier() + '.'
          + this.identifier;
    return this.qualifiedIdentifier;
  }

  public boolean isDefined() {
    return this.isDefined;
  }

  public void makeDefined() {
    this.isDefined = true;
  }

  public ClassAndInterfaceList getUsedClassAndInterfaceList() {
    return this.usedTypes;
  }

  public ClassAndInterfaceList getUserClassAndInterfaceList() {
    return this.userTypes;
  }
}
