/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collections;

import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.IdentifiedPackage;
import org.xmloperator.java.skeleton.impl.PackageImpl;

public final class IdentifiedPackageImpl extends PackageImpl
    implements IdentifiedPackage {
  protected final IdentifiedPackage parentPackage;
  protected final String identifier;
  protected String qualifiedIdentifier = null;
  protected final List childPackageList = new ArrayList();

  public IdentifiedPackageImpl(IdentifiedPackage parentPackage,
                               String identifier) {
    if (identifier.length() == 0 || identifier.indexOf('.') >= 0)
      throw new IllegalArgumentException();
    this.parentPackage = parentPackage;
    this.identifier = identifier;
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    return object instanceof IdentifiedPackage
        && this.getQualifiedIdentifier().
        equals(((IdentifiedPackage)object).getQualifiedIdentifier());
  }

  /////////////////////////
  // Package implementation
  /////////////////////////

  public boolean isDefault() {
    return false;
  }

  public boolean hasClassOrInterface(boolean isDefined) {
    if (super.hasClassOrInterface(isDefined))
      return true;
    for (int k = 0; k < this.childPackageList.size(); k++)
      if (((IdentifiedPackage)this.childPackageList.get(k)).
          hasClassOrInterface(isDefined))
        return true;
    return false;
  }

  ///////////////////////////////////
  // IdentifiedPackage implementation
  ///////////////////////////////////

  public IdentifiedPackage getParent() {
    return this.parentPackage;
  }

  public String getIdentifier() {
    return identifier;
  }

  public String getQualifiedIdentifier() {
    if (this.qualifiedIdentifier == null)
      this.qualifiedIdentifier = this.parentPackage == null ? this.identifier :
          this.parentPackage.getQualifiedIdentifier() + '.' + this.identifier;
    return this.qualifiedIdentifier;
  }

  public int getChildPackageCount() {
    return this.childPackageList.size();
  }

  public IdentifiedPackage getChildPackage(int index) {
    return (IdentifiedPackage)this.childPackageList.get(index);
  }

  public boolean addChildPackage(IdentifiedPackage childPackage) {
    if (childPackage.getParent() != this)
      throw new IllegalArgumentException();
    if (this.childPackageList.contains(childPackage))
      return false;
    this.childPackageList.add(childPackage);
    return true;
  }

  public IdentifiedPackage getChildPackage(String identifier) {
    for (int k = 0; k < this.childPackageList.size(); k++) {
      IdentifiedPackage childPackage =
          (IdentifiedPackage)this.childPackageList.get(k);
      if (childPackage.getIdentifier().equals(identifier))
        return childPackage;
    }
    return null;
  }

  public IdentifiedPackage addChildPackage(String identifier) {
    IdentifiedPackage childPackage = this.getChildPackage(identifier);
    if (childPackage != null)
      return childPackage;
    childPackage =
        new IdentifiedPackageImpl( (IdentifiedPackage)this, identifier);
    this.childPackageList.add(childPackage);
    return childPackage;
  }

  public IdentifiedPackage getDescendantPackage(String qualifiedIdentifier) {
    int dotPos = qualifiedIdentifier.indexOf('.');
    String childIdentifier = dotPos < 0 ? qualifiedIdentifier :
        qualifiedIdentifier.substring(0, dotPos);
    IdentifiedPackage childPackage = this.getChildPackage(childIdentifier);
    return dotPos < 0 || childPackage == null ? childPackage :
        childPackage.getDescendantPackage(
        qualifiedIdentifier.substring(dotPos + 1));
  }

  public IdentifiedPackage addDescendantPackage(String qualifiedIdentifier) {
    int dotPos = qualifiedIdentifier.indexOf('.');
    String childIdentifier = dotPos < 0 ? qualifiedIdentifier :
        qualifiedIdentifier.substring(0, dotPos);
    IdentifiedPackage childPackage = this.addChildPackage(childIdentifier);
    return dotPos < 0 ? childPackage :
        childPackage.addDescendantPackage(
        qualifiedIdentifier.substring(dotPos + 1));
  }

  public void sort() {
    this.sortClassesAndInterfaces();
    this.sortChildPackages();
    for (int k = 0; k < this.childPackageList.size(); k++)
      ((IdentifiedPackage)this.childPackageList.get(k)).sort();
  }

  private void sortChildPackages() {
    Collections.sort(this.childPackageList,
                     new Comparator() {
      public int compare(Object o1, Object o2) {
        return ((IdentifiedPackage)o1).getIdentifier().compareTo(
            ( (IdentifiedPackage)o2).getIdentifier());
      }
    });
  }
}
