/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.model;

import org.xmloperator.java.skeleton.model.Package;

/**
 * An IdentifiedPackage object represents an identified Java package, such as
 *   "org.xmloperator.java".
 */
public interface IdentifiedPackage extends Package {

  /**
   * Return the parent package if any.
   * Null for a primary package like "org" or "com".
   */
  public IdentifiedPackage getParent();

  /**
   * Return the package identifier. Not null neither empty.
   */
  public String getIdentifier();

  /**
   * Return the package qualified identifier. Not null neither empty.
   * If parent is not null then the qualified identifier is :
   *   <parent qualified identifier> . <identifier>
   * For a primary package the qualified identifier is the identifier.
   */
  public String getQualifiedIdentifier();

  /**
   * Return the count of child package.
   */
  public int getChildPackageCount();

  /**
   * Return the child package with the given index (0..count-1).
   */
  public IdentifiedPackage getChildPackage(int index);

  /**
   * Add a child package to the child package list.
   * Return true in case of success, false if a child package with the same
   *   identifier already exists.
   */
  public boolean addChildPackage(IdentifiedPackage childPackage);

  /**
   * Return the child package with the given identifier if it exists,
   *   null elsewhere.
   */
  public IdentifiedPackage getChildPackage(String identifier);

  /**
   * Add a child package with the given identifier to the child package
   *   list and return it.
   * Return the child package with this identifier if already existing.
   */
  public IdentifiedPackage addChildPackage(String identifier);

  /**
   * Return the descendant package with the given qualified identifier
   *   if it exists, null elsewhere.
   * The qualified identifier is the part relative to this package.
   */
  public IdentifiedPackage getDescendantPackage(String qualifiedIdentifier);

  /**
   * Add if necessary a descendant package and return it.
   */
  public IdentifiedPackage addDescendantPackage(String qualifiedIdentifier);

  /**
   * Sorts the classes and interfaces by lexicographic order of identifier.
   * In each class, sorts the used classes and interfaces.
   * Sorts the child packages  by lexicographic order of identifier.
   * Sorts recursively the descendants packages.
   */
  public void sort();
}
