/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.model;

import org.xmloperator.java.skeleton.model.ClassOrInterface;

/**
 * A Package object represents a Java package.
 * It may be an identified package like "org.xmloperator.java" or the program
 *   default package.
 */
public interface Package {

  /**
   * Return true for a default package, false for an identified package.
   */
  public boolean isDefault();

  /**
   * Return the class and interface count of the package.
   */
  public int getClassAndInterfaceCount();

  /**
   * Return the class or interface of the package with the given index
   * (0..count-1).
   */
  public ClassOrInterface getClassOrInterface(int index);

  /**
   * Add a class or interface to the class and interface list.
   * Return true in case of success, false if a class or interface with the same
   *   identifier already exists.
   */
  public boolean addClassOrInterface(ClassOrInterface classOrInterface);

  /**
   * Return the class or interface with the given identifier if it exists,
   *   null elsewhere.
   */
  public ClassOrInterface getClassOrInterface(String identifier);

  /**
   * Add a class or interface with the given identifier to the class and
   *   interface list and return it.
   * Return the class or interface with this identifier if already existing.
   */
  public ClassOrInterface addClassOrInterface(String identifier);

  /**
   * Sorts the classes and interfaces by lexicographic order of identifier.
   * In each class, sorts the used classes and interfaces.
   */
  public void sortClassesAndInterfaces();

  /**
   * Return true if this package (or any sub-package) has a class or
   *   interface that satisfy the condition given by the argument,
   *   false elsewhere.
   */
  public boolean hasClassOrInterface(boolean isDefined);
}
