/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.skeleton.model;

import org.xmloperator.java.skeleton.model.Package;
import org.xmloperator.java.skeleton.model.IdentifiedPackage;

/**
 * A Skeleton object represents the skeleton of a Java program.
 * The skeleton is only concerned by classes and interfaces dependencies.
 * Imbedded classes and interfaces are ignored.
 */
public interface Skeleton {

  /**
   * Return the program default package.
   */
  public Package getDefaultPackage();

  /**
   * Return the count of primary package (such as "org" or "com").
   */
  public int getPrimaryPackageCount();

  /**
   * Return the primary package with the given index (0..count-1).
   */
  public IdentifiedPackage getPrimaryPackage(int index);

  /**
   * Add a primary package to the primary package list.
   * Return true in case of success, false if a primary package with the same
   *   identifier already exists.
   */
  public boolean addPrimaryPackage(IdentifiedPackage primaryPackage);

  /**
   * Return the primary package with the given identifier if it exists,
   *   null elsewhere.
   */
  public IdentifiedPackage getPrimaryPackage(String identifier);

  /**
   * Add if necessary a primary package with the given identifier
   *   to the primary package list and return it.
   */
  public IdentifiedPackage addPrimaryPackage(String identifier);

  /**
   * Return the package with the given qualified identifier
   *   if it exists, null elsewhere.
   */
  public IdentifiedPackage getPackage(String qualifiedIdentifier);

  /**
   * Add if necessary a package with the given qualified identifier
   *   and return it.
   */
  public IdentifiedPackage addPackage(String qualifiedIdentifier);

  /**
   * Return the class or interface with the given qualified identifier
   *   if it exists, null elsewhere.
   */
  public ClassOrInterface getClassOrInterface(String qualifiedIdentifier);

  /**
   * Add if necessary a class or interface with the given qualified identifier
   *   and return it.
   */
  public ClassOrInterface addClassOrInterface(String qualifiedIdentifier);

  /**
   * Sorts the packages by lexicographic order of identifier.
   * In each package, sorts the classes and interfaces by lexicographic order
   *   of identifier.
   */
  public void sort();
}
