/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.syntax;

public interface JavaToken {

  /**
   * If this token is defined then return its code else return -1.
   * Type codes belongs to {TYPE_*}.
   */
  public int getType();
  public static final int NO_TOKEN = -1;
  public static final int TYPE_IDENTIFIER = 0;
  public static final int TYPE_KEYWORD = 1;
  public static final int TYPE_LITERAL = 2;
  public static final int TYPE_SEPARATOR = 3;
  public static final int TYPE_OPERATOR = 4;

  /**
   * Return the token value.
   */
  public String getValue();

  /**
   * Return true if the token value equals the given value.
   */
  public boolean valueEquals(String value);

  /**
   * Return the length of the token value.
   */
  public int getValueLength();

  /**
   * Copy the token value to <code>chars</code> from <code>start</code> position.
   */
  public void copyValueTo(char[] chars, int start);

  /**
   * Append the token value to the string buffer.
   */
  public void appendValueTo(StringBuffer sb);

  /**
   * If this token is a keyword then return its code else -1.
   * A keyword code belongs to {KEYWORD_*}.
   */
  public int getKeyword();
  public static final int NOT_A_KEYWORD = -1;
  public static final int KEYWORD_ABSTRACT = 0; // "abstract"
  public static final int KEYWORD_BOOLEAN = 1; // "boolean"
  public static final int KEYWORD_BREAK = 2; // "break"
  public static final int KEYWORD_BYTE = 3; // "byte"
  public static final int KEYWORD_CASE = 4; // "case"
  public static final int KEYWORD_CATCH = 5; // "catch"
  public static final int KEYWORD_CHAR = 6; // "char"
  public static final int KEYWORD_CLASS = 7; // "class"
  public static final int KEYWORD_CONST = 8; // "const"
  public static final int KEYWORD_CONTINUE = 9; // "continue"
  public static final int KEYWORD_DEFAULT = 10; // "default"
  public static final int KEYWORD_DO = 11; // "do"
  public static final int KEYWORD_DOUBLE = 12; // "double"
  public static final int KEYWORD_ELSE = 13; // "else"
  public static final int KEYWORD_EXTENDS = 14; // "extends"
  public static final int KEYWORD_FINAL = 15; // "final"
  public static final int KEYWORD_FINALLY = 16; // "finally"
  public static final int KEYWORD_FLOAT = 17; // "float"
  public static final int KEYWORD_FOR = 18; // "for"
  public static final int KEYWORD_GOTO = 19; // "goto"
  public static final int KEYWORD_IF = 20; // "if"
  public static final int KEYWORD_IMPLEMENTS = 21; // "implements"
  public static final int KEYWORD_IMPORT = 22; // "import"
  public static final int KEYWORD_INSTANCEOF = 23; // "instanceof"
  public static final int KEYWORD_INT = 24; // "int"
  public static final int KEYWORD_INTERFACE = 25; // "interface"
  public static final int KEYWORD_LONG = 26; // "long"
  public static final int KEYWORD_NATIVE = 27; // "native"
  public static final int KEYWORD_NEW = 28; // "new"
  public static final int KEYWORD_PACKAGE = 29; // "package"
  public static final int KEYWORD_PRIVATE = 30; // "private"
  public static final int KEYWORD_PROTECTED = 31; // "protected"
  public static final int KEYWORD_PUBLIC = 32; // "public"
  public static final int KEYWORD_RETURN = 33; // "return"
  public static final int KEYWORD_SHORT = 34; // "short"
  public static final int KEYWORD_STATIC = 35; // "static"
  public static final int KEYWORD_STRICTFTP = 36; // "strictftp"
  public static final int KEYWORD_SUPER = 37; // "super"
  public static final int KEYWORD_SWITCH = 38; // "switch"
  public static final int KEYWORD_SYNCHRONIZED = 39; // "synchronized"
  public static final int KEYWORD_THIS = 40; // "this"
  public static final int KEYWORD_THROW = 41; // "throw"
  public static final int KEYWORD_THROWS = 42; // "throws"
  public static final int KEYWORD_TRANSIENT = 43; // "transient"
  public static final int KEYWORD_TRY = 44; // "try"
  public static final int KEYWORD_VOID = 45; // "void"
  public static final int KEYWORD_VOLATILE = 46; // "volatile"
  public static final int KEYWORD_WHILE = 47; // "while"
  public static final int KEYWORD_ASSERT = 48; // "assert"
  public static final int KEYWORD_COUNT = 49;

  /**
   * If this token is a separator then return its code else -1.
   * A separator code belongs to {SEPARATOR_*}.
   */
  public int getSeparator();
  public static final int NOT_A_SEPARATOR = -1;
  public static final int SEPARATOR_START_ROUND_BRACKET = 0; // '('
  public static final int SEPARATOR_END_ROUND_BRACKET = 1; // ')'
  public static final int SEPARATOR_START_BRACE_BRACKET = 2; // '{'
  public static final int SEPARATOR_END_BRACE_BRACKET = 3; // '}'
  public static final int SEPARATOR_START_SQUARE_BRACKET = 4; // '['
  public static final int SEPARATOR_END_SQUARE_BRACKET = 5; // ']'
  public static final int SEPARATOR_SEMI_COLON = 6; // ';'
  public static final int SEPARATOR_COMMA = 7; // ','
  public static final int SEPARATOR_DOT = 8; // '.'
  public static final int SEPARATOR_COUNT = 9;

  /**
   * If this token is an operator then return its code else -1.
   * An operator code belongs to {OPERATOR_*}.
   */
  public int getOperator();
  public static final int NOT_AN_OPERATOR = -1;
  public static final int OPERATOR_ASSIGNMENT = 0; // "="
  public static final int OPERATOR_GREATER = 1; // ">"
  public static final int OPERATOR_LESSER = 2; // "<"
  public static final int OPERATOR_NOT = 3; // "!"
  public static final int OPERATOR_OPPOSITE = 4; // "~"
  public static final int OPERATOR_QUESTION = 5; // "?"
  public static final int OPERATOR_COLON = 6; // ":"
  public static final int OPERATOR_EQUAL = 7; // "=="
  public static final int OPERATOR_LESSER_OR_EQUAL = 8; // "<="
  public static final int OPERATOR_GREATER_OR_EQUAL = 9; // ">="
  public static final int OPERATOR_NOT_EQUAL = 10; // "!="
  public static final int OPERATOR_OPTIONAL_AND = 11; // "&&"
  public static final int OPERATOR_OPTIONAL_OR = 12; // "||"
  public static final int OPERATOR_INCREMENT = 13; // "++"
  public static final int OPERATOR_DECREMENT = 14; // "--"
  public static final int OPERATOR_ADD = 15; // "+"
  public static final int OPERATOR_SUB = 16; // "-"
  public static final int OPERATOR_MULTIPLY = 17; // "*"
  public static final int OPERATOR_DIVIDE = 18; // "/"
  public static final int OPERATOR_REQUIRED_AND = 19; // "&"
  public static final int OPERATOR_REQUIRED_OR = 20; // "|"
  public static final int OPERATOR_XOR = 21; // "^"
  public static final int OPERATOR_MODULO = 22; // "%"
  public static final int OPERATOR_LEFT_SHIFT = 23; // "<<"
  public static final int OPERATOR_RIGHT_SHIFT = 24; // ">>"
  public static final int OPERATOR_UNSIGNED_RIGHT_SHIFT = 25; // ">>>"
  public static final int OPERATOR_ADD_ASSIGNMENT = 26; // "+="
  public static final int OPERATOR_SUB_ASSIGNMENT = 27; // "-="
  public static final int OPERATOR_MULTIPLY_ASSIGNMENT = 28; // "*="
  public static final int OPERATOR_DIVIDE_ASSIGNMENT = 29; // "/="
  public static final int OPERATOR_AND_ASSIGNMENT = 30; // "&="
  public static final int OPERATOR_OR_ASSIGNMENT = 31; // "|="
  public static final int OPERATOR_XOR_ASSIGNMENT = 32; // "^="
  public static final int OPERATOR_MODULO_ASSIGNMENT = 33; // "%="
  public static final int OPERATOR_LEFT_SHIFT_ASSIGNMENT = 34; // "<<="
  public static final int OPERATOR_RIGHT_SHIFT_ASSIGNMENT = 35; // ">>="
  public static final int OPERATOR_UNSIGNED_RIGHT_SHIFT_ASSIGNMENT = 36; // ">>>="
  public static final int OPERATOR_COUNT = 37;

  /**
   * Copy the given java token.
   */
  void copy(JavaToken javaToken);
}
