/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.syntax;

import org.xmloperator.java.syntax.JavaToken;

public class JavaTokenImpl implements JavaToken {
  private static final String[] keywords =
      {"abstract", "boolean", "break", "byte", "case", "catch", "char", "class",
      "const", "continue", "default", "do", "double", "else", "extends",
      "final", "finally", "float", "for", "goto", "if", "implements", "import",
      "instanceof", "int", "interface", "long", "native", "new", "package",
      "private", "protected", "public", "return", "short", "static", "strictfp",
      "super", "switch", "synchronized", "this", "throw", "throws", "transient",
      "try", "void", "volatile", "while", "assert"};
  private static final String[] separators = {
      "(", ")", "{", "}", "[", "]", ";", ",", "."};
  private static final String[] operators = {
      "=", ">", "<", "!", "~", "?", ":", "==", "<=", ">=", "!=",
      "&&", "||", "++", "--", "+", "-", "*", "/", "&", "|", "^", "%",
      "<<", ">>", ">>>", "+=", "-=", "*=", "/=", "&=", "|=", "^=", "%=",
      "<<=", ">>=", ">>>="};
  private int type = NO_TOKEN;
  private char[] buffer;
  private int length = 0;
  private String value = null;
  private int keywordOrSeparatorOrOperator = 0;

  /**
   * Constructor. NO_TOKEN.
   */
  public JavaTokenImpl() {
    this(1024);
  }

  /**
   * Constructor. NO_TOKEN.
   */
  public JavaTokenImpl(int bufferSize) {
    this.buffer = new char[bufferSize];
  }

  /**
   * NO_TOKEN.
   */
  public void clear() {
    this.type = NO_TOKEN;
    this.length = 0;
    this.value = null;
  }

  public void setIdentifier(char[] chars, int start, int length) {
    this.type = TYPE_IDENTIFIER;
    if (length > this.buffer.length)
      this.buffer = new char[length];
    System.arraycopy(chars, start, this.buffer, 0, length);
    this.length = length;
    this.value = null;
  }

  public void setKeyword(int keyword) {
    this.type = TYPE_KEYWORD;
    this.length = 0;
    this.value = null;
    this.keywordOrSeparatorOrOperator = keyword;
  }

  public void setLiteral(char[] chars, int start, int length) {
    this.type = TYPE_LITERAL;
    if (length > this.buffer.length)
      this.buffer = new char[length];
    System.arraycopy(chars, start, this.buffer, 0, length);
    this.length = length;
    this.value = null;
  }

  public void setSeparator(int separator) {
    this.type = TYPE_SEPARATOR;
    this.length = 0;
    this.value = null;
    this.keywordOrSeparatorOrOperator = separator;
  }

  public void setOperator(int operator) {
    this.type = TYPE_OPERATOR;
    this.length = 0;
    this.value = null;
    this.keywordOrSeparatorOrOperator = operator;
  }

  ///////////////////////////
  // JavaToken implementation
  ///////////////////////////

  public int getType() {
    return this.type;
  }

  public String getValue() {
    if (this.value == null)
      switch (this.type) {
        case TYPE_IDENTIFIER:
        case TYPE_LITERAL:
          this.value = new String(this.buffer, 0, this.length);
          break;
        case TYPE_KEYWORD:
          this.value = keywords[this.keywordOrSeparatorOrOperator];
          break;
        case TYPE_SEPARATOR:
          this.value = separators[this.keywordOrSeparatorOrOperator];
          break;
        case TYPE_OPERATOR:
          this.value = operators[this.keywordOrSeparatorOrOperator];
          break;
        default:
          this.value = "";
      }
    return this.value;
  }

  public boolean valueEquals(String value) {
    switch (this.type) {
      case TYPE_IDENTIFIER:
      case TYPE_LITERAL:
        if (value.length() != this.length)
          return false;
        for (int k = 0; k < this.length; k++)
          if (value.charAt(k) != this.buffer[k])
            return false;
        return true;
      case TYPE_KEYWORD:
        return value.equals(keywords[this.keywordOrSeparatorOrOperator]);
      case TYPE_SEPARATOR:
        return value.equals(separators[this.keywordOrSeparatorOrOperator]);
      case TYPE_OPERATOR:
        return value.equals(operators[this.keywordOrSeparatorOrOperator]);
      default:
        return false;
    }
  }

  public int getValueLength() {
    if (this.length == 0)
      switch (this.type) {
        case TYPE_KEYWORD:
          this.length = keywords[this.keywordOrSeparatorOrOperator].length();
          break;
        case TYPE_SEPARATOR:
          this.length = separators[this.keywordOrSeparatorOrOperator].length();
          break;
        case TYPE_OPERATOR:
          this.length = operators[this.keywordOrSeparatorOrOperator].length();
          break;
      }
    return this.length;
  }

  public void copyValueTo(char[] chars, int start) {
    String value;
    switch (this.type) {
      case TYPE_IDENTIFIER:
      case TYPE_LITERAL:
        System.arraycopy(this.buffer, 0, chars, start, this.length);
        return;
      case TYPE_KEYWORD:
        value = keywords[this.keywordOrSeparatorOrOperator];
        break;
      case TYPE_SEPARATOR:
        value = separators[this.keywordOrSeparatorOrOperator];
        break;
      case TYPE_OPERATOR:
        value = operators[this.keywordOrSeparatorOrOperator];
        break;
      default:
        return;
    }
    value.getChars(0, value.length(), chars, start);
  }

  public void appendValueTo(StringBuffer sb) {
    String value;
    switch (this.type) {
      case TYPE_IDENTIFIER:
      case TYPE_LITERAL:
        sb.append(this.buffer, 0, this.length);
        return;
      case TYPE_KEYWORD:
        value = keywords[this.keywordOrSeparatorOrOperator];
        break;
      case TYPE_SEPARATOR:
        value = separators[this.keywordOrSeparatorOrOperator];
        break;
      case TYPE_OPERATOR:
        value = operators[this.keywordOrSeparatorOrOperator];
        break;
      default:
        return;
    }
    sb.append(value);
  }

  public int getKeyword() {
    return this.type == TYPE_KEYWORD ?
        this.keywordOrSeparatorOrOperator : NOT_A_KEYWORD;
  }

  public int getSeparator() {
    return this.type == TYPE_SEPARATOR ?
        this.keywordOrSeparatorOrOperator : NOT_A_SEPARATOR;
  }

  public int getOperator() {
    return this.type == TYPE_OPERATOR ?
        this.keywordOrSeparatorOrOperator : NOT_AN_OPERATOR;
  }

  public void copy(JavaToken javaToken) {
    this.value = null;
    if (javaToken instanceof JavaTokenImpl) {
      this.type = ((JavaTokenImpl)javaToken).type;
      this.length = ((JavaTokenImpl)javaToken).length;
      if (this.buffer.length < this.length)
        this.buffer = new char[this.length];
      System.arraycopy(((JavaTokenImpl)javaToken).buffer, 0,
                       this.buffer, 0, this.length);
      this.keywordOrSeparatorOrOperator =
          ((JavaTokenImpl)javaToken).keywordOrSeparatorOrOperator;
    }
    else {
      this.type = javaToken.getType();
      this.length = javaToken.getValueLength();
      if (this.buffer.length < this.length)
        this.buffer = new char[this.length];
      javaToken.copyValueTo(this.buffer, 0);
      switch (this.type) {
        case TYPE_KEYWORD:
          this.keywordOrSeparatorOrOperator = javaToken.getKeyword();
          break;
        case TYPE_SEPARATOR:
          this.keywordOrSeparatorOrOperator = javaToken.getSeparator();
          break;
        case TYPE_OPERATOR:
          this.keywordOrSeparatorOrOperator = javaToken.getOperator();
          break;
      }
    }
  }
}
