/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
/// Java is a trademark of Sun Microsystems, Inc.
package org.xmloperator.java.syntax;

import java.util.Arrays;

import org.xmloperator.java.syntax.JavaToken;

public abstract class JavaTokenUtil {
  private static final boolean[] MODIFIERS =
      new boolean[JavaToken.KEYWORD_COUNT];
  private static final boolean[] BASIC_TYPES =
      new boolean[JavaToken.KEYWORD_COUNT];
  private static final boolean[] ASSIGNMENTS =
      new boolean[JavaToken.OPERATOR_COUNT];
  private static final boolean[] INFIXES =
      new boolean[JavaToken.OPERATOR_COUNT];
  private static final boolean[] PREFIXES =
      new boolean[JavaToken.OPERATOR_COUNT];

  static {
    Arrays.fill(MODIFIERS, false);
    MODIFIERS[JavaToken.KEYWORD_ABSTRACT] = true;
    MODIFIERS[JavaToken.KEYWORD_FINAL] = true;
    MODIFIERS[JavaToken.KEYWORD_NATIVE] = true;
    MODIFIERS[JavaToken.KEYWORD_PRIVATE] = true;
    MODIFIERS[JavaToken.KEYWORD_PROTECTED] = true;
    MODIFIERS[JavaToken.KEYWORD_PUBLIC] = true;
    MODIFIERS[JavaToken.KEYWORD_STATIC] = true;
    MODIFIERS[JavaToken.KEYWORD_STRICTFTP] = true;
    MODIFIERS[JavaToken.KEYWORD_SYNCHRONIZED] = true;
    MODIFIERS[JavaToken.KEYWORD_TRANSIENT] = true;
    MODIFIERS[JavaToken.KEYWORD_VOLATILE] = true;

    Arrays.fill(BASIC_TYPES, false);
    BASIC_TYPES[JavaToken.KEYWORD_BOOLEAN] = true;
    BASIC_TYPES[JavaToken.KEYWORD_BYTE] = true;
    BASIC_TYPES[JavaToken.KEYWORD_CHAR] = true;
    BASIC_TYPES[JavaToken.KEYWORD_DOUBLE] = true;
    BASIC_TYPES[JavaToken.KEYWORD_FLOAT] = true;
    BASIC_TYPES[JavaToken.KEYWORD_INT] = true;
    BASIC_TYPES[JavaToken.KEYWORD_LONG] = true;
    BASIC_TYPES[JavaToken.KEYWORD_SHORT] = true;

    Arrays.fill(ASSIGNMENTS, false);
    ASSIGNMENTS[JavaToken.OPERATOR_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_ADD_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_SUB_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_MULTIPLY_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_DIVIDE_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_AND_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_OR_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_XOR_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_MODULO_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_LEFT_SHIFT_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_RIGHT_SHIFT_ASSIGNMENT] = true;
    ASSIGNMENTS[JavaToken.OPERATOR_UNSIGNED_RIGHT_SHIFT_ASSIGNMENT] = true;

    Arrays.fill(INFIXES, false);
    INFIXES[JavaToken.OPERATOR_ADD] = true;
    INFIXES[JavaToken.OPERATOR_DIVIDE] = true;
    INFIXES[JavaToken.OPERATOR_EQUAL] = true;
    INFIXES[JavaToken.OPERATOR_GREATER] = true;
    INFIXES[JavaToken.OPERATOR_GREATER_OR_EQUAL] = true;
    INFIXES[JavaToken.OPERATOR_LEFT_SHIFT] = true;
    INFIXES[JavaToken.OPERATOR_LESSER] = true;
    INFIXES[JavaToken.OPERATOR_LESSER_OR_EQUAL] = true;
    INFIXES[JavaToken.OPERATOR_MODULO] = true;
    INFIXES[JavaToken.OPERATOR_MULTIPLY] = true;
    INFIXES[JavaToken.OPERATOR_NOT_EQUAL] = true;
    INFIXES[JavaToken.OPERATOR_OPTIONAL_AND] = true;
    INFIXES[JavaToken.OPERATOR_OPTIONAL_OR] = true;
    INFIXES[JavaToken.OPERATOR_REQUIRED_AND] = true;
    INFIXES[JavaToken.OPERATOR_REQUIRED_OR] = true;
    INFIXES[JavaToken.OPERATOR_RIGHT_SHIFT] = true;
    INFIXES[JavaToken.OPERATOR_SUB] = true;
    INFIXES[JavaToken.OPERATOR_UNSIGNED_RIGHT_SHIFT] = true;
    INFIXES[JavaToken.OPERATOR_XOR] = true;

    Arrays.fill(PREFIXES, false);
    PREFIXES[JavaToken.OPERATOR_ADD] = true;
    PREFIXES[JavaToken.OPERATOR_DECREMENT] = true;
    PREFIXES[JavaToken.OPERATOR_INCREMENT] = true;
    PREFIXES[JavaToken.OPERATOR_NOT] = true;
    PREFIXES[JavaToken.OPERATOR_OPPOSITE] = true;
    PREFIXES[JavaToken.OPERATOR_SUB] = true;
  }

  /**
   * Return true if this token is one of the following keywords :
   * public protected private static abstract final native synchronized
   * transient volatile strictfp
   */
  public static final boolean isModifier(int keyword) {
    return keyword >= 0 && MODIFIERS[keyword];
  }

  /**
   * Return true if this token is one of the following keywords :
   * byte short char int long float double boolean
   */
  public static final boolean isBasicType(int keyword) {
    return keyword >= 0 && BASIC_TYPES[keyword];
  }

  /**
   * Return true if this token is one of the assignment operators.
   */
  public static final boolean isAssignmentOperator(int operator) {
    return operator >= 0 && ASSIGNMENTS[operator];
  }

  /**
   * Return true if this token is one of the infix operators.
   */
  public static final boolean isInfixOperator(int operator) {
    return operator >= 0 && INFIXES[operator];
  }

  /**
   * Return true if this token is one of the prefix operators.
   */
  public static final boolean isPrefixOperator(int operator) {
    return operator >= 0 && PREFIXES[operator];
  }

  /**
   * Return true if this token is one of the postfix operators.
   */
  public static final boolean isPostfixOperator(int operator) {
    return operator == JavaToken.OPERATOR_DECREMENT
        || operator == JavaToken.OPERATOR_INCREMENT;
  }
}
